// VisualClassBuilderView.cpp : implementation of the CVisualClassBuilderView class
//

#include "stdafx.h"
#include "VisualClassBuilder.h"

#include "VisualClassBuilderDoc.h"
#include "VisualClassBuilderView.h"
#include "ClassDlg.h"
#include "StructDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView

IMPLEMENT_DYNCREATE(CVisualClassBuilderView, CView)

BEGIN_MESSAGE_MAP(CVisualClassBuilderView, CView)
	//{{AFX_MSG_MAP(CVisualClassBuilderView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView construction/destruction

CVisualClassBuilderView::CVisualClassBuilderView()
{
	m_bPrinting = FALSE;
}

CVisualClassBuilderView::~CVisualClassBuilderView()
{
	// Destroy memory DC
	if (m_hMemDC)
	{
		::SelectObject(m_hMemDC, m_hOldMemBitmap);
		::DeleteDC(m_hMemDC);
		m_hMemDC = NULL;
		::DeleteObject(m_hMemBitmap);
		m_hMemBitmap = NULL;
		m_hOldMemBitmap = NULL;
	}
}

BOOL CVisualClassBuilderView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView drawing

void CVisualClassBuilderView::OnDraw(CDC* pDC)
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Clear background
	RECT clientRect;
	GetClientRect(&clientRect);
	FillRect(m_hMemDC, &clientRect, (HBRUSH)GetStockObject(WHITE_BRUSH));

	// Draw object list
	pDoc->m_VCBObjectManager.DrawVCBObjects(m_hMemDC);
		
	if (!m_bPrinting)
	{
		// Blit on screen DC
		BitBlt(pDC->m_hDC, 0, 0, clientRect.right-clientRect.left, clientRect.bottom-clientRect.top, m_hMemDC, 0, 0, SRCCOPY);
	}
	else
	{
		// Blit on printer DC
		StretchBlt(pDC->m_hDC, 0, 0, m_PageWidth, m_PageHeight, m_hMemDC, 0, 0, clientRect.right-clientRect.left, clientRect.bottom-clientRect.top, SRCCOPY);
	}
}

void CVisualClassBuilderView::OnInitialUpdate()
{
	RECT clientRect;
	GetClientRect(&clientRect);

	// Create memory DC
	HDC hDC = ::GetDC(NULL);
	m_hMemDC = CreateCompatibleDC(hDC);
	m_hMemBitmap = CreateCompatibleBitmap(hDC, clientRect.right-clientRect.left, clientRect.bottom-clientRect.top);
	m_hOldMemBitmap = (HBITMAP)::SelectObject(m_hMemDC, m_hMemBitmap);
	::ReleaseDC(NULL, hDC);

	CView::OnInitialUpdate();
}

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView printing

BOOL CVisualClassBuilderView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	BOOL bResult = DoPreparePrinting(pInfo);

	if (bResult)
	{
		// Set landscape page orientation
		DEVMODE* pDevMode = (DEVMODE*)::GlobalLock(pInfo->m_pPD->m_pd.hDevMode);
		pDevMode->dmPaperSize = DMPAPER_A4;
		pDevMode->dmOrientation = DMORIENT_LANDSCAPE;
		ResetDC(pInfo->m_pPD->m_pd.hDC, pDevMode);
		::GlobalUnlock(pInfo->m_pPD->m_pd.hDevMode);
	}

	return bResult;
}

void CVisualClassBuilderView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	m_bPrinting = TRUE;
}

void CVisualClassBuilderView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	m_bPrinting = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView diagnostics

#ifdef _DEBUG
void CVisualClassBuilderView::AssertValid() const
{
	CView::AssertValid();
}

void CVisualClassBuilderView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVisualClassBuilderDoc* CVisualClassBuilderView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVisualClassBuilderDoc)));
	return (CVisualClassBuilderDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVisualClassBuilderView message handlers

void CVisualClassBuilderView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Save start point
	m_StartPoint = (POINT)point;

	// Select object
	pDoc->m_VCBObjectManager.SelectVCBObject(m_StartPoint);

	// Refresh screen
	Invalidate(FALSE);
	
	CView::OnLButtonDown(nFlags, point);
}

void CVisualClassBuilderView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Save end point
	m_EndPoint = (POINT)point;

	// Check for selected object
	if (pDoc->m_VCBObjectManager.GetCurrentVCBObject())
	{
		// Move selected object
		int offsetX = m_EndPoint.x - m_StartPoint.x;
		int offsetY = m_EndPoint.y - m_StartPoint.y;
		pDoc->m_VCBObjectManager.GetCurrentVCBObject()->MoveObject(offsetX, offsetY);

		// Refresh screen
		Invalidate(FALSE);
	}
	
	CView::OnLButtonUp(nFlags, point);
}

void CVisualClassBuilderView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Check if left mouse button down
	if (nFlags == MK_LBUTTON)
	{
		// Save end point
		m_EndPoint = (POINT)point;

		// Check for selected object
		if (pDoc->m_VCBObjectManager.GetCurrentVCBObject())
		{
			// Move selected object
			int offsetX = m_EndPoint.x - m_StartPoint.x;
			int offsetY = m_EndPoint.y - m_StartPoint.y;
			pDoc->m_VCBObjectManager.GetCurrentVCBObject()->MoveObject(offsetX, offsetY);

			// Set document modified flag
			pDoc->SetModifiedFlag(TRUE);

			// Save start point
			m_StartPoint = m_EndPoint;

			// Refresh screen
			Invalidate(FALSE);
		}
	}
	
	CView::OnMouseMove(nFlags, point);
}

void CVisualClassBuilderView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Check for pressed key
	switch (nChar)
	{
		case VK_DELETE:
		{
			// Remove object
			pDoc->m_VCBObjectManager.RemoveVCBObject(pDoc->m_VCBObjectManager.GetCurrentVCBObject());

			// Refresh screen
			Invalidate(FALSE);
		}
		break;
	}
	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CVisualClassBuilderView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CVisualClassBuilderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Check for selected object
	if (pDoc->m_VCBObjectManager.GetCurrentVCBObject())
	{
		switch (pDoc->m_VCBObjectManager.GetCurrentVCBObject()->GetVCBObjectType())
		{
			case OBJ_CLASS:
			{
				CVCBClass* pVCBClass = (CVCBClass*)pDoc->m_VCBObjectManager.GetCurrentVCBObject();
				CClassDlg classDlg;

				// Get class params
				_tcscpy(classDlg.m_lpszClassName, pVCBClass->GetVCBClassName());
				_tcscpy(classDlg.m_lpszClassHeaderFile, pVCBClass->GetVCBClassHeaderFile());
				_tcscpy(classDlg.m_lpszClassImplementationFile, pVCBClass->GetVCBClassImplementationFile());
				_tcscpy(classDlg.m_lpszAuthor, pVCBClass->GetVCBClassAuthor());
				_tcscpy(classDlg.m_lpszDescription, pVCBClass->GetVCBClassDescription());
				classDlg.m_pVariableList = pVCBClass->GetVCBClassVariables();
				classDlg.m_iVariableNumber = pVCBClass->GetVCBClassVariableNumber();
				classDlg.m_pMethodList = pVCBClass->GetVCBClassMethods();
				classDlg.m_iMethodNumber = pVCBClass->GetVCBClassMethodNumber();
				classDlg.m_pParentList = pVCBClass->GetVCBClassParents();
				classDlg.m_iParentNumber = pVCBClass->GetVCBClassParentNumber();
				classDlg.m_bMFCHeaders = pVCBClass->GetVCBClassMFCHeaders();
				classDlg.m_bComments = pVCBClass->GetVCBClassComments();

				// Get classes
				CVCBObject* pVCBObject;
				for (int i=0; i<pDoc->m_VCBObjectManager.GetVCBObjectNumber(); i++)
				{
					pVCBObject = pDoc->m_VCBObjectManager.GetVCBObject(i);
					if (pVCBObject->GetVCBObjectType() == OBJ_CLASS)
					{
						CVCBClass* pVCBTempClass = (CVCBClass*)pVCBObject;
						if (pVCBTempClass->GetVCBObjectID() != pVCBClass->GetVCBObjectID())
						{
							classDlg.m_iClassNumber++;
							if (!classDlg.m_pClassList)
								classDlg.m_pClassList = (PARENT_NODE*)malloc(classDlg.m_iClassNumber*sizeof(PARENT_NODE));
							else
								classDlg.m_pClassList = (PARENT_NODE*)realloc(classDlg.m_pClassList, classDlg.m_iClassNumber*sizeof(PARENT_NODE));
							_tcscpy(classDlg.m_pClassList[classDlg.m_iClassNumber-1].lpszClassName, pVCBTempClass->GetVCBClassName());
							_tcscpy(classDlg.m_pClassList[classDlg.m_iClassNumber-1].lpszClassHeaderFile, pVCBTempClass->GetVCBClassHeaderFile());
							classDlg.m_pClassList[classDlg.m_iClassNumber-1].classAccessLevel = AL_PUBLIC;
						}
					}
				}

				if (classDlg.DoModal() == IDOK)
				{
					// Set class params
					pVCBClass->SetVCBClassName(classDlg.m_lpszClassName);
					pVCBClass->SetVCBClassHeaderFile(classDlg.m_lpszClassHeaderFile);
					pVCBClass->SetVCBClassImplementationFile(classDlg.m_lpszClassImplementationFile);
					pVCBClass->SetVCBClassAuthor(classDlg.m_lpszAuthor);
					pVCBClass->SetVCBClassDescription(classDlg.m_lpszDescription);
					pVCBClass->SetVCBClassVariableNumber(classDlg.m_iVariableNumber);
					pVCBClass->SetVCBClassVariables(classDlg.m_pVariableList);
					pVCBClass->SetVCBClassMethodNumber(classDlg.m_iMethodNumber);
					pVCBClass->SetVCBClassMethods(classDlg.m_pMethodList);
					pVCBClass->SetVCBClassParentNumber(classDlg.m_iParentNumber);
					pVCBClass->SetVCBClassParents(classDlg.m_pParentList);
					pVCBClass->SetVCBClassMFCHeaders(classDlg.m_bMFCHeaders);
					pVCBClass->SetVCBClassComments(classDlg.m_bComments);
					
					// Refresh screen
					Invalidate(FALSE);
				}

				// Clear class list
				if (classDlg.m_pClassList)
				{
					free(classDlg.m_pClassList);
					classDlg.m_pClassList = NULL;
					classDlg.m_iClassNumber = 0;
				}
			}
			break;

			case OBJ_STRUCT:
			{
				CVCBStruct* pVCBStruct = (CVCBStruct*)pDoc->m_VCBObjectManager.GetCurrentVCBObject();
				CStructDlg structDlg;

				// Get struct params
				_tcscpy(structDlg.m_lpszStructName, pVCBStruct->GetVCBStructName());
				_tcscpy(structDlg.m_lpszAuthor, pVCBStruct->GetVCBStructAuthor());
				_tcscpy(structDlg.m_lpszDescription, pVCBStruct->GetVCBStructDescription());
				structDlg.m_pVariableList = pVCBStruct->GetVCBStructVariables();
				structDlg.m_iVariableNumber = pVCBStruct->GetVCBStructVariableNumber();

				if (structDlg.DoModal() == IDOK)
				{
					// Set struct params
					pVCBStruct->SetVCBStructName(structDlg.m_lpszStructName);
					pVCBStruct->SetVCBStructAuthor(structDlg.m_lpszAuthor);
					pVCBStruct->SetVCBStructDescription(structDlg.m_lpszDescription);
					pVCBStruct->SetVCBStructVariableNumber(structDlg.m_iVariableNumber);
					pVCBStruct->SetVCBStructVariables(structDlg.m_pVariableList);
					
					// Refresh screen
					Invalidate(FALSE);
				}
			}
			break;
		}
	}

	CView::OnLButtonDblClk(nFlags, point);
}

void CVisualClassBuilderView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	// Destroy memory DC
	if (m_hMemDC)
	{
		::SelectObject(m_hMemDC, m_hOldMemBitmap);
		::DeleteDC(m_hMemDC);
		::DeleteObject(m_hMemBitmap);
	}
	
	// Create memory DC
	HDC hDC = ::GetDC(NULL);
	m_hMemDC = CreateCompatibleDC(hDC);
	m_hMemBitmap = CreateCompatibleBitmap(hDC, cx, cy);
	m_hOldMemBitmap = (HBITMAP)::SelectObject(m_hMemDC, m_hMemBitmap);
	::ReleaseDC(NULL, hDC);
}

BOOL CVisualClassBuilderView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	return /*CView::OnEraseBkgnd(pDC)*/FALSE;
}

void CVisualClassBuilderView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pInfo)
	{
		m_PageWidth = GetDeviceCaps(pDC->m_hAttribDC, PHYSICALWIDTH) - 2*GetDeviceCaps(pDC->m_hAttribDC, PHYSICALOFFSETX);
		m_PageHeight = GetDeviceCaps(pDC->m_hAttribDC, PHYSICALHEIGHT) - 2*GetDeviceCaps(pDC->m_hAttribDC, PHYSICALOFFSETY);
	}
	
	CView::OnPrepareDC(pDC, pInfo);
}
