// VCBObjectManager.cpp: implementation of the CVCBObjectManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VCBObjectManager.h"



CVCBObjectManager::CVCBObjectManager()
{
	// Init members
	m_pVCBObjectList = NULL;
	m_iVCBObjectNumber = 0;
	m_pCurrentObject = NULL;
	m_dwVCBObjectID = 0;
}

CVCBObjectManager::~CVCBObjectManager()
{
	// Clear object list
	ClearVBCObjectList();
}

void CVCBObjectManager::ClearVBCObjectList()
{
	// Remove all objects from the object list
	VCBOBJ_LIST *pCurr, *pPrev;
	pCurr = m_pVCBObjectList;
	pPrev = NULL;
	while (pCurr)
	{
		pPrev = pCurr;
		pCurr = pCurr->pNext;

		// Delete object
		delete pPrev->pVCBObject;
		delete pPrev;
		pPrev = NULL;
	}

	m_pVCBObjectList = NULL;
	m_iVCBObjectNumber = 0;
	m_pCurrentObject = NULL;
	m_dwVCBObjectID = 0;
}

void CVCBObjectManager::AddVCBObject(CVCBObject* pVCBObject)
{
	// Increment object id counter
	m_dwVCBObjectID++;

	// Create new object
	VCBOBJ_LIST* pNewObject = new VCBOBJ_LIST;
	pNewObject->pVCBObject = pVCBObject;
	pNewObject->pVCBObject->SetVCBObjectID(m_dwVCBObjectID);
	pNewObject->pNext = NULL;

	// Add new object to the list
	if (!m_pVCBObjectList)
		m_pVCBObjectList = pNewObject;
	else
	{
		// Find last object in the list
		VCBOBJ_LIST *pCurr = m_pVCBObjectList;
		while (pCurr->pNext)
			pCurr = pCurr->pNext;

		pCurr->pNext = pNewObject;
	}

	// Increment object number
	m_iVCBObjectNumber++;

	// Set current object
	m_pCurrentObject = pNewObject->pVCBObject;
}

void CVCBObjectManager::RemoveVCBObject(CVCBObject* pVCBObject)
{
	// Check for valid object list
	if (m_pVCBObjectList)
	{
		// Remove object from the object list
		VCBOBJ_LIST *pCurr, *pPrev;
		pCurr = m_pVCBObjectList;
		pPrev = NULL;
		while (pCurr)
		{
			// Check object
			if (pCurr->pVCBObject == pVCBObject)
			{
				// Delete object
				if (!pPrev)
					m_pVCBObjectList = m_pVCBObjectList->pNext;
				else
					pPrev->pNext = pCurr->pNext;

				delete pCurr->pVCBObject;
				delete pCurr;
				pCurr = NULL;

				// Decrement object number
				m_iVCBObjectNumber--;

				break;
			}
			else
			{
				pPrev = pCurr;
				pCurr = pCurr->pNext;
			}
		}
	}
}

int CVCBObjectManager::GetVCBObjectNumber()
{
	// Get object number
	return m_iVCBObjectNumber;
}

CVCBObject* CVCBObjectManager::GetCurrentVCBObject()
{
	// Get current object
	return m_pCurrentObject;
}

void CVCBObjectManager::SetCurrentVCBObject(DWORD id)
{
	// Check for valid object list
	if (m_pVCBObjectList)
	{
		// Find object in the list
		VCBOBJ_LIST *pCurr = m_pVCBObjectList;
		while (pCurr)
		{
			// Check object ID
			if (pCurr->pVCBObject->GetVCBObjectID() == id)
			{
				// Set current object
				m_pCurrentObject = pCurr->pVCBObject;

				break;
			}
			else
				pCurr = pCurr->pNext;
		}
	}
}

CVCBObject* CVCBObjectManager::GetVCBObject(int index)
{
	CVCBObject* pVCBObject = NULL;

	// Check for valid object list
	if (m_pVCBObjectList)
	{
		VCBOBJ_LIST* pCurr = m_pVCBObjectList;
		int objectIndex = 0;
		while (pCurr)
		{
			if (objectIndex == index)
			{
				// Get object
				pVCBObject = pCurr->pVCBObject;

				break;
			}
			else
			{
				objectIndex++;
				pCurr = pCurr->pNext;
			}
		}
	}

	return pVCBObject;
}

void CVCBObjectManager::SelectVCBObject(POINT pt)
{
	// Check for valid object list
	if (m_pVCBObjectList)
	{
		// Clear selected object
		m_pCurrentObject = NULL;

		// Find object in the list
		VCBOBJ_LIST *pCurr = m_pVCBObjectList;
		while (pCurr)
		{
			// Check object selection
			pCurr->pVCBObject->SelectVCBObject(pt);
			if (pCurr->pVCBObject->IsVCBObjectSelected())
			{
				// Check for previously selected object
				if (m_pCurrentObject)
				{
					// Unselect object
					POINT pt = {-1, -1};
					m_pCurrentObject->SelectVCBObject(pt);
				}

				// Set current object
				m_pCurrentObject = pCurr->pVCBObject;
			}

			pCurr = pCurr->pNext;
		}
	}
}

void CVCBObjectManager::DrawVCBObjects(HDC hDC)
{
	// Check for valid object list
	if (m_pVCBObjectList)
	{
		// Draw object list
		VCBOBJ_LIST *pCurr = m_pVCBObjectList;
		while (pCurr)
		{
			// Draw object
			pCurr->pVCBObject->DrawVCBObject(hDC);

			// Check for class object
			if (pCurr->pVCBObject->GetVCBObjectType() == OBJ_CLASS)
			{
				// Draw class inheritance relation
				CVCBClass* pVCBClass = (CVCBClass*)pCurr->pVCBObject;

				VCBOBJ_LIST *pTarget = m_pVCBObjectList;
				while (pTarget)
				{
					// Check for class object
					if (pTarget->pVCBObject->GetVCBObjectType() == OBJ_CLASS)
					{
						CVCBClass* pChildClass = (CVCBClass*)pTarget->pVCBObject;

						RECT parentRect = pVCBClass->GetVCBObjectRect();
						POINT parentStartPoint = {(parentRect.left+parentRect.right)/2, parentRect.bottom+SHADOW_DISTANCE/2};
						POINT parentInheritancePolygon[3] = {{parentStartPoint.x-10, parentStartPoint.y+20}, {parentStartPoint.x, parentStartPoint.y}, {parentStartPoint.x+10, parentStartPoint.y+20}};

						// Check for object ID
						if (pChildClass->GetVCBObjectID() != pVCBClass->GetVCBObjectID())
						{
							if (pChildClass->GetVCBClassParentNumber() > 0)
							{
								bool found = false;
								PARENT_NODE* pParents = pChildClass->GetVCBClassParents();
								for (int i=0; i<pChildClass->GetVCBClassParentNumber(); i++)
								{
									if (_tcscmp(pParents[i].lpszClassName, pVCBClass->GetVCBClassName()) == 0)
									{
										// Draw child class polyline
										RECT childRect = pChildClass->GetVCBObjectRect();
										POINT childStartPoint = {(childRect.left+childRect.right)/2, childRect.top};

										if ((parentStartPoint.y+20) < childStartPoint.y)
										{
											POINT middlePoint1 = {childStartPoint.x, (childStartPoint.y+(parentStartPoint.y+20))/2};
											POINT middlePoint2 = {parentStartPoint.x, (childStartPoint.y+(parentStartPoint.y+20))/2};
											POINT childInheritancePolyline[4] = {{childStartPoint.x, childStartPoint.y}, {middlePoint1.x, middlePoint1.y}, {middlePoint2.x, middlePoint2.y}, {parentStartPoint.x, parentStartPoint.y+20}};
											Polyline(hDC, childInheritancePolyline, 4);
										}
										else
										{
											POINT middlePoint1 = {childStartPoint.x, childStartPoint.y-10};
											POINT middlePoint2 = {(childStartPoint.x+parentStartPoint.x)/2, childStartPoint.y-10};
											POINT middlePoint3 = {(childStartPoint.x+parentStartPoint.x)/2, parentStartPoint.y+30};
											POINT middlePoint4 = {parentStartPoint.x, parentStartPoint.y+30};											
											POINT childInheritancePolyline[6] = {{childStartPoint.x, childStartPoint.y}, {middlePoint1.x, middlePoint1.y}, {middlePoint2.x, middlePoint2.y}, {middlePoint3.x, middlePoint3.y}, {middlePoint4.x, middlePoint4.y}, {parentStartPoint.x, parentStartPoint.y+20}};
											Polyline(hDC, childInheritancePolyline, 6);
										}

										found = true;
									}
								}

								if (found)
								{
									// Draw parent class polygon
									Polygon(hDC, parentInheritancePolygon, 3);
								}
							}
						}
					}

					pTarget = pTarget->pNext;
				}
			}

			pCurr = pCurr->pNext;
		}
	}
}

void CVCBObjectManager::GenerateOutputFiles()
{
	// Check for valid object list
	if (m_pVCBObjectList)
	{
		// Generate output files
		VCBOBJ_LIST *pCurr = m_pVCBObjectList;
		while (pCurr)
		{
			// Generate object output
			pCurr->pVCBObject->GenerateOutputFile();

			pCurr = pCurr->pNext;
		}
	}
}

DWORD CVCBObjectManager::GetLastObjectID()
{
	// Get last object ID
	return m_dwVCBObjectID;
}

void CVCBObjectManager::SaveToArchive(CArchive& ar)
{
	// Save project settings
	ar << m_iVCBObjectNumber;

	// Save objects to the project file
	VCBOBJ_LIST *pCurr = m_pVCBObjectList;
	while (pCurr)
	{
		// Save object
		pCurr->pVCBObject->SaveObject(ar);

		pCurr = pCurr->pNext;
	}
}

void CVCBObjectManager::LoadFromArchive(CArchive& ar)
{
	int i, j;
	int numberObjects;
	int objectType;
	DWORD objectID;
	RECT objectRect;
	CString strClassName, strHeaderFile, strImplementationFile, strClassAuthor, strClassDescription;
	CString strVariableType, strVariableName, strVariableComment, strPropertyName;
	CString strMethodType, strMethodName, strMethodComment;
	CString strParentName, strParentFile;
	int numberVariables, variableAccessLevel, numberMethods, methodAccessLevel, numberParents, parentAccessLevel;
	BOOL bMFCHeaders, bComments;

	// Clear object list
	ClearVBCObjectList();
	
	// Load number of objects
	ar >> numberObjects;

	// Load objects
	for (i=0; i<numberObjects; i++)
	{
		// Load object type
		ar >> objectType;

		// Check object type
		switch ((VCBOBJ_TYPE)objectType)
		{
			case OBJ_CLASS:
			{
				// Load CVCBClass object
				ar >> objectID;
				ar >> objectRect;
				ar >> strClassName;
				ar >> strHeaderFile;
				ar >> strImplementationFile;
				ar >> strClassAuthor;
				ar >> strClassDescription;
				ar >> numberVariables;
				VARIABLE_TYPE* pVariableList = NULL;
				if (numberVariables > 0)
					pVariableList = (VARIABLE_TYPE*)malloc(numberVariables*sizeof(VARIABLE_TYPE));
				for (j=0; j<numberVariables; j++)
				{
					ar >> strVariableType;
					_tcscpy(pVariableList[j].lpszVariableType, strVariableType);
					ar >> strVariableName;
					_tcscpy(pVariableList[j].lpszVariableName, strVariableName);
					ar >> strVariableComment;
					_tcscpy(pVariableList[j].lpszVariableComment, strVariableComment);
					ar >> pVariableList[j].bProperty;
					ar >> strPropertyName;
					_tcscpy(pVariableList[j].lpszPropertyName, strPropertyName);
					ar >> pVariableList[j].bStatic;
					ar >> variableAccessLevel;
					pVariableList[j].variableAccessLevel = (ACCESS_LEVEL)variableAccessLevel;
				}
				ar >> numberMethods;
				METHOD_TYPE* pMethodList = NULL;
				if (numberMethods > 0)
					pMethodList = (METHOD_TYPE*)malloc(numberMethods*sizeof(METHOD_TYPE));
				for (j=0; j<numberMethods; j++)
				{
					ar >> strMethodType;
					_tcscpy(pMethodList[j].lpszMethodType, strMethodType);
					ar >> strMethodName;
					_tcscpy(pMethodList[j].lpszMethodName, strMethodName);
					ar >> strMethodComment;
					_tcscpy(pMethodList[j].lpszMethodComment, strMethodComment);
					ar >> pMethodList[j].bVirtual;
					ar >> pMethodList[j].bStatic;
					ar >> methodAccessLevel;
					pMethodList[j].methodAccessLevel = (ACCESS_LEVEL)methodAccessLevel;
				}
				ar >> numberParents;
				PARENT_NODE* pParentList = NULL;
				if (numberParents > 0)
					pParentList = (PARENT_NODE*)malloc(numberParents*sizeof(PARENT_NODE));
				for (j=0; j<numberParents; j++)
				{
					ar >> strParentName;
					_tcscpy(pParentList[j].lpszClassName, strParentName);
					ar >> parentAccessLevel;
					pParentList[j].classAccessLevel = (ACCESS_LEVEL)parentAccessLevel;
					ar >> strParentFile;
					_tcscpy(pParentList[j].lpszClassHeaderFile, strParentFile);
				}
				ar >> bMFCHeaders;
				ar >> bComments;

				// Create new CVCBClass object
				CVCBClass* pNewVCBClass = new CVCBClass(objectRect);
				AddVCBObject(pNewVCBClass);
				pNewVCBClass->SetVCBObjectID(objectID);
				pNewVCBClass->SetVCBClassName((LPTSTR)((LPCTSTR)strClassName));
				pNewVCBClass->SetVCBClassHeaderFile((LPTSTR)((LPCTSTR)strHeaderFile));
				pNewVCBClass->SetVCBClassImplementationFile((LPTSTR)((LPCTSTR)strImplementationFile));
				pNewVCBClass->SetVCBClassAuthor((LPTSTR)((LPCTSTR)strClassAuthor));
				pNewVCBClass->SetVCBClassDescription((LPTSTR)((LPCTSTR)strClassDescription));
				pNewVCBClass->SetVCBClassVariableNumber(numberVariables);
				pNewVCBClass->SetVCBClassVariables(pVariableList);
				pNewVCBClass->SetVCBClassMethodNumber(numberMethods);
				pNewVCBClass->SetVCBClassMethods(pMethodList);
				pNewVCBClass->SetVCBClassParentNumber(numberParents);
				pNewVCBClass->SetVCBClassParents(pParentList);
				pNewVCBClass->SetVCBClassMFCHeaders(bMFCHeaders);
				pNewVCBClass->SetVCBClassComments(bComments);
			}
			break;

			case OBJ_STRUCT:
			{
				// Load CVCBStruct object
				ar >> objectID;
				ar >> objectRect;
				ar >> strClassName;
				ar >> strHeaderFile;
				ar >> strClassAuthor;
				ar >> strClassDescription;
				ar >> numberVariables;
				VARIABLE_TYPE* pVariableList = NULL;
				if (numberVariables > 0)
					pVariableList = (VARIABLE_TYPE*)malloc(numberVariables*sizeof(VARIABLE_TYPE));
				for (j=0; j<numberVariables; j++)
				{
					ar >> strVariableType;
					_tcscpy(pVariableList[j].lpszVariableType, strVariableType);
					ar >> strVariableName;
					_tcscpy(pVariableList[j].lpszVariableName, strVariableName);
					ar >> strVariableComment;
					_tcscpy(pVariableList[j].lpszVariableComment, strVariableComment);
					ar >> pVariableList[j].bProperty;
					ar >> strPropertyName;
					_tcscpy(pVariableList[j].lpszPropertyName, strPropertyName);
					ar >> pVariableList[j].bStatic;
					ar >> variableAccessLevel;
					pVariableList[j].variableAccessLevel = (ACCESS_LEVEL)variableAccessLevel;
				}

				// Create new CVCBStruct object
				CVCBStruct* pNewVCBStruct = new CVCBStruct(objectRect);
				AddVCBObject(pNewVCBStruct);
				pNewVCBStruct->SetVCBObjectID(objectID);
				pNewVCBStruct->SetVCBStructName((LPTSTR)((LPCTSTR)strClassName));
				pNewVCBStruct->SetVCBStructHeaderFile((LPTSTR)((LPCTSTR)strHeaderFile));
				pNewVCBStruct->SetVCBStructAuthor((LPTSTR)((LPCTSTR)strClassAuthor));
				pNewVCBStruct->SetVCBStructDescription((LPTSTR)((LPCTSTR)strClassDescription));
				pNewVCBStruct->SetVCBStructVariableNumber(numberVariables);
				pNewVCBStruct->SetVCBStructVariables(pVariableList);
			}
			break;
		}
	}
}
