// VCBStruct.cpp: implementation of the CVCBStruct class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VCBStruct.h"
#include "../utils/Gradient.h"



CVCBStruct::CVCBStruct(RECT rect)
{
	// Init members
	m_nObjectType = OBJ_STRUCT;
	m_rcObjectRect = rect;
	_tcscpy(m_lpszStructName, _T("MY_STRUCT"));
	_tcscpy(m_lpszStructHeaderFile, _T("Includes.h"));
	_tcscpy(m_lpszAuthor, _T(""));
	_tcscpy(m_lpszDescription, _T(""));
	m_pVariableList = NULL;
	m_iVariableNumber = 0;

	// Init object fonts
	int fontHeight;
	fontHeight = -MulDiv(10, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72);
	m_hStructNameFont = CreateFont(fontHeight, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Times New Roman"));
	fontHeight = -MulDiv(8, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72);
	m_hAttributesFont = CreateFont(fontHeight, 0, 0, 0, FW_NORMAL, TRUE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Times New Roman"));

	// Get screen DC
	HDC hDC = GetDC(NULL);

	// Init memory DC and bitmap
	m_hMemDC = CreateCompatibleDC(hDC);
	m_hMemBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldMemBitmap = (HBITMAP)SelectObject(m_hMemDC, m_hMemBitmap);

	// Init shadow DC and bitmap
	m_hShadowDC = CreateCompatibleDC(hDC);
	m_hShadowBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldShadowBitmap = (HBITMAP)SelectObject(m_hShadowDC, m_hShadowBitmap);

	// Init background DC and bitmap
	m_hBackgroundDC = CreateCompatibleDC(hDC);
	m_hBackgroundBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldBackgroundBitmap = (HBITMAP)SelectObject(m_hBackgroundDC, m_hBackgroundBitmap);

	// Draw object shadow
	DrawShadow();

	// Draw object background
	DrawBackground();

	// Release screen DC
	ReleaseDC(NULL, hDC);
}

CVCBStruct::~CVCBStruct()
{
	// Delete object fonts
	if (m_hStructNameFont)
	{
		DeleteObject(m_hStructNameFont);
		m_hStructNameFont = NULL;
	}
	if (m_hAttributesFont)
	{
		DeleteObject(m_hAttributesFont);
		m_hAttributesFont = NULL;
	}

	// Check for valid variable list
	if (m_pVariableList)
	{
		// Destroy variable list
		free(m_pVariableList);
		m_pVariableList = NULL;
		m_iVariableNumber = 0;
	}
}

void CVCBStruct::DrawVCBObject(HDC hDC)
{
	int i;
	CString strText;

	// Draw object shadow
	BitBlt(m_hMemDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, m_hShadowDC, 0, 0, SRCCOPY);

	// Draw object background
	BitBlt(m_hMemDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE, m_hBackgroundDC, 0, 0, SRCCOPY);

	int oldBkMode = SetBkMode(m_hMemDC, TRANSPARENT);

	// Draw struct name
	COLORREF oldTextColor = SetTextColor(m_hMemDC, RGB(128,0,0));
	HFONT hOldFont = (HFONT)SelectObject(m_hMemDC, m_hStructNameFont);
	RECT structNameRect = {5, 5, m_rcObjectRect.right-m_rcObjectRect.left-10, 20};
	DrawText(m_hMemDC, m_lpszStructName, _tcslen(m_lpszStructName), &structNameRect, DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_WORD_ELLIPSIS);

	// Draw struct members
	SetTextColor(m_hMemDC, RGB(0,0,128));
	SelectObject(m_hMemDC, m_hAttributesFont);
	RECT memberRect = {5, 27, m_rcObjectRect.right-m_rcObjectRect.left-5, 42};
	for (i=0; i<m_iVariableNumber; i++)
	{
		strText.Format(_T("%s : %s"), m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableType);
		DrawText(m_hMemDC, strText, _tcslen(strText), &memberRect, DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_WORD_ELLIPSIS);
		OffsetRect(&memberRect, 0, 15);

		// Check for full members list
		if (memberRect.top > 100)
			break;
	}

	SetTextColor(m_hMemDC, RGB(0,0,128));
	SelectObject(m_hMemDC, hOldFont);
	SetBkMode(m_hMemDC, oldBkMode);

	// Draw object selection
	if (m_bIsSelected)
	{
		LOGBRUSH lb;
		lb.lbStyle = BS_HATCHED;
		lb.lbHatch = HS_FDIAGONAL;
		lb.lbColor = RGB(0,0,128);
		HPEN hSelectionPen = ExtCreatePen(PS_GEOMETRIC, 3, &lb, 0, NULL);
		HPEN hOldPen = (HPEN)SelectObject(m_hMemDC, hSelectionPen);
		HBRUSH hOldBrush = (HBRUSH)SelectObject(m_hMemDC, GetStockObject(HOLLOW_BRUSH));
		Rectangle(m_hMemDC, 1, 1, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE);
		SelectObject(m_hMemDC, hOldBrush);
		SelectObject(m_hMemDC, hOldPen);
		DeleteObject(hSelectionPen);
	}

	// Update transparency
	UpdateTransparency(hDC);

	// Blit on screenDC
	BitBlt(hDC, m_rcObjectRect.left, m_rcObjectRect.top, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, m_hMemDC, 0, 0, SRCCOPY);
}

void CVCBStruct::DrawShadow()
{
	// Clear shadow DC background
	HBRUSH hBgBrush = CreateSolidBrush(RGB(255,255,255));
	RECT bgRect = {0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top};
	FillRect(m_hShadowDC, &bgRect, hBgBrush);
	DeleteObject(hBgBrush);

	int shadowDistance = SHADOW_DISTANCE / 2;
	int shadowWidth = abs(m_rcObjectRect.right - m_rcObjectRect.left - shadowDistance);
	int shadowHeight = abs(m_rcObjectRect.bottom - m_rcObjectRect.top - shadowDistance);
	RECT shadowRect = {0, 0, shadowWidth, shadowHeight};
	HBRUSH hShadowBgBrush = CreateSolidBrush(RGB(255,255,255));
	HBRUSH hShadowBrush = CreateSolidBrush(RGB(0,0,0));
	int filterSize = FILTER_SIZE;

	if ((shadowWidth > filterSize) && (shadowHeight > filterSize))
	{
		HDC hScreenDC = GetDC(NULL);
		HDC hShadowDC = CreateCompatibleDC(hScreenDC);
		HBITMAP hShadowBitmap = CreateCompatibleBitmap(hScreenDC, shadowWidth, shadowHeight);
		HBITMAP hOldShadowBitmap = (HBITMAP)SelectObject(hShadowDC, hShadowBitmap);
		ReleaseDC(NULL, hScreenDC);

		// Draw shadow rectangle
		FillRect(hShadowDC, &shadowRect, hShadowBgBrush);
		InflateRect(&shadowRect, -shadowDistance, -shadowDistance);
		FillRect(hShadowDC, &shadowRect, hShadowBrush);

		BITMAP shadowBMP;
		GetObject(hShadowBitmap, sizeof(BITMAP), &shadowBMP);
		int size = shadowBMP.bmWidthBytes * shadowBMP.bmHeight;
		int bpp = shadowBMP.bmBitsPixel / 8;
		UCHAR* pShadowBits = new UCHAR[size];
		UCHAR* pTempShadowBits = new UCHAR[size];
		GetBitmapBits(hShadowBitmap, size, pShadowBits);
		memcpy(pTempShadowBits, pShadowBits, size);
		int red, green, blue;
		int imageOffsetY = 0;
		int imageOffsetX = 0;
		int imageOffset;
		int filterOffsetX;
		int filterOffsetY;
		int filterOffset;
		int filterHalfSize = filterSize / 2;
		BOOL bLeftEdge, bTopEdge, bRightEdge, bBottomEdge;
		for (int i=0; i<shadowBMP.bmHeight; i++)
		{
			imageOffsetX = 0;
			for (int j=0; j<shadowBMP.bmWidth; j++)
			{
				bLeftEdge = (j <= filterSize-1);
				bTopEdge = (i <= filterSize-1);
				bRightEdge =  (j >= shadowBMP.bmWidth-filterSize+1);
				bBottomEdge = (i >= shadowBMP.bmHeight-filterSize+1);

				if (bLeftEdge || bTopEdge || bRightEdge || bBottomEdge)
				{
					red = green = blue = 0;
					for (int k=-filterHalfSize; k<filterHalfSize; k++)
					{
						filterOffsetY = min(max(0, i+k), shadowBMP.bmHeight-1) * shadowBMP.bmWidthBytes;
						for (int l=-filterHalfSize; l<filterHalfSize; l++)
						{
							filterOffsetX = min(max(0, j+l), shadowBMP.bmWidth) * bpp;
							filterOffset = filterOffsetY + filterOffsetX;
							
							blue += pShadowBits[filterOffset];
							green += pShadowBits[filterOffset+1];
							red += pShadowBits[filterOffset+2];

							filterOffsetX += bpp;
						}

						filterOffsetY += shadowBMP.bmWidthBytes;
					}

					imageOffset = imageOffsetY + imageOffsetX;

					if ((imageOffset+2) < size)
					{
						blue = blue >> filterSize;
						green = green >> filterSize;
						red = red >> filterSize;
						pTempShadowBits[imageOffset] = blue;
						pTempShadowBits[imageOffset+1] = green;
						pTempShadowBits[imageOffset+2] = red;
					}
				}

				imageOffsetX += bpp;
			}

			imageOffsetY += shadowBMP.bmWidthBytes;
		}

		SetBitmapBits(hShadowBitmap, size, pTempShadowBits);
		delete pTempShadowBits;
		delete pShadowBits;

		BitBlt(m_hShadowDC, shadowDistance, shadowDistance, shadowWidth, shadowHeight, hShadowDC, 0, 0, SRCCOPY);

		DeleteObject(hShadowBgBrush);
		DeleteObject(hShadowBrush);

		if (hShadowDC)
		{
			SelectObject(hShadowDC, hOldShadowBitmap);
			DeleteDC(hShadowDC);
			DeleteObject(hShadowBitmap);
		}
	}
}

void CVCBStruct::DrawBackground()
{
	// Clear object background
	HBRUSH hBgBrush = CreateSolidBrush(RGB(255,255,255));
	RECT bgRect = {0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top};
	FillRect(m_hMemDC, &bgRect, hBgBrush);
	DeleteObject(hBgBrush);

	// Draw object background
	CGradient gradient;
	RECT rect = {bgRect.left, bgRect.top, bgRect.right-3, bgRect.bottom-3};
	gradient.VerticalGradient(m_hBackgroundDC, &rect, RGB(200,255,255), RGB(255,255,255), TRUE, 1.4);
	HPEN hPen = CreatePen(PS_SOLID, 1, RGB(128,0,0));
	HPEN hOldPen = (HPEN)SelectObject(m_hBackgroundDC, hPen);
	HBRUSH hOldBrush = (HBRUSH)SelectObject(m_hBackgroundDC, GetStockObject(HOLLOW_BRUSH));
	Rectangle(m_hBackgroundDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE);
	Rectangle(m_hBackgroundDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, 25);
	SelectObject(m_hBackgroundDC, hOldBrush);
	SelectObject(m_hBackgroundDC, hOldPen);
	DeleteObject(hPen);
}

void CVCBStruct::UpdateTransparency(HDC hDC)
{
	int i, j;
	int imageOffset, imageOffsetX, imageOffsetY;

	HDC hScreenDC = GetDC(NULL);
	HDC hTempDC = CreateCompatibleDC(hScreenDC);
	HBITMAP hTempBitmap = CreateCompatibleBitmap(hScreenDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	HBITMAP hOldTempBitmap = (HBITMAP)SelectObject(hTempDC, hTempBitmap);
	ReleaseDC(NULL, hScreenDC);

	// Blit from screen DC
	BitBlt(hTempDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, hDC, m_rcObjectRect.left, m_rcObjectRect.top, SRCCOPY);

	// Get screen data
	BITMAP tempBMP;
	GetObject(hTempBitmap, sizeof(BITMAP), &tempBMP);
	int size = tempBMP.bmWidthBytes * tempBMP.bmHeight;
	int bpp = tempBMP.bmBitsPixel / 8;
	UCHAR* pTempBits = new UCHAR[size];
	GetBitmapBits(hTempBitmap, size, pTempBits);

	// Get memory data
	UCHAR* pMemBits = new UCHAR[size];
	GetBitmapBits(m_hMemBitmap, size, pMemBits);

	// Update shadow transparency
	imageOffsetY = 0;
	imageOffsetX = 0;
	for (i=0; i<tempBMP.bmHeight; i++)
	{
		imageOffsetX = 0;
		for (j=0; j<tempBMP.bmWidth; j++)
		{
			imageOffset = imageOffsetY + imageOffsetX;

			if ((imageOffset+2) < size)
			{
				// Check for full-transparent region
				if ((((i>=tempBMP.bmHeight-SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=0) && (j<SHADOW_DISTANCE))) || 
					(((i>=0) && (i<SHADOW_DISTANCE)) && ((j>=tempBMP.bmWidth-SHADOW_DISTANCE) && (j<tempBMP.bmWidth))))
				{
					pMemBits[imageOffset] = pTempBits[imageOffset];
					pMemBits[imageOffset+1] = pTempBits[imageOffset+1];
					pMemBits[imageOffset+2] = pTempBits[imageOffset+2];
				}
				// Check for semi-transparent region
				else if ((((i>=SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=tempBMP.bmWidth-SHADOW_DISTANCE) && (j<tempBMP.bmWidth))) || 
					(((i>=tempBMP.bmHeight-SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=SHADOW_DISTANCE) && (j<tempBMP.bmWidth))))
				{
					pMemBits[imageOffset] = (UCHAR)max(0, min(255, (pMemBits[imageOffset]*SHADOW_TRANSPARENCY + pTempBits[imageOffset]*(1-SHADOW_TRANSPARENCY))));
					pMemBits[imageOffset+1] = (UCHAR)max(0, min(255, (UCHAR)(pMemBits[imageOffset+1]*SHADOW_TRANSPARENCY + pTempBits[imageOffset+1]*(1-SHADOW_TRANSPARENCY))));
					pMemBits[imageOffset+2] = (UCHAR)max(0, min(255, (UCHAR)(pMemBits[imageOffset+2]*SHADOW_TRANSPARENCY + pTempBits[imageOffset+2]*(1-SHADOW_TRANSPARENCY))));
				}
			}

			imageOffsetX += bpp;
		}

		imageOffsetY += tempBMP.bmWidthBytes;
	}

	SetBitmapBits(m_hMemBitmap, size, pMemBits);
	delete pTempBits;
	delete pMemBits;

	if (hTempDC)
	{
		SelectObject(hTempDC, hOldTempBitmap);
		DeleteDC(hTempDC);
		DeleteObject(hTempBitmap);
	}
}

void CVCBStruct::SelectVCBObject(POINT pt)
{
	// Check if object is selected
	m_bIsSelected = PtInRect(&m_rcObjectRect, pt);
}

RECT CVCBStruct::GetVCBObjectRect()
{
	// Get object rectangle
	return m_rcObjectRect;
}

void CVCBStruct::MoveObject(int offsetX, int offsetY)
{
	// Move object
	m_rcObjectRect.left += offsetX;
	m_rcObjectRect.top += offsetY;
	m_rcObjectRect.right += offsetX;
	m_rcObjectRect.bottom += offsetY;
}

_TCHAR* CVCBStruct::GetVCBStructName()
{
	// Get struct name
	return m_lpszStructName;
}

void CVCBStruct::SetVCBStructName(_TCHAR* lpszStructName)
{
	// Check for valid struct name
	if (_tcscmp(lpszStructName, _T("")))
	{
		// Set struct name
		_tcscpy(m_lpszStructName, lpszStructName);
	}
}

_TCHAR* CVCBStruct::GetVCBStructHeaderFile()
{
	// Get header file
	return m_lpszStructHeaderFile;
}

void CVCBStruct::SetVCBStructHeaderFile(_TCHAR* lpszStructHeaderFile)
{
	// Check for valid header file
	if (_tcscmp(lpszStructHeaderFile, _T("")))
	{
		// Set struct header file
		_tcscpy(m_lpszStructHeaderFile, lpszStructHeaderFile);
	}
}

_TCHAR* CVCBStruct::GetVCBStructAuthor()
{
	// Get author
	return m_lpszAuthor;
}

void CVCBStruct::SetVCBStructAuthor(_TCHAR* lpszAuthor)
{
	// Check for valid author
	if (_tcscmp(lpszAuthor, _T("")))
	{
		// Set author
		_tcscpy(m_lpszAuthor, lpszAuthor);
	}
}

_TCHAR* CVCBStruct::GetVCBStructDescription()
{
	// Get description
	return m_lpszDescription;
}

void CVCBStruct::SetVCBStructDescription(_TCHAR* lpszDescription)
{
	// Check for valid description
	if (_tcscmp(lpszDescription, _T("")))
	{
		// Set description
		_tcscpy(m_lpszDescription, lpszDescription);
	}
}

int CVCBStruct::GetVCBStructVariableNumber()
{
	// Get variable number
	return m_iVariableNumber;
}

void CVCBStruct::SetVCBStructVariableNumber(int number)
{
	// Set variable number
	m_iVariableNumber = number;
}

VARIABLE_TYPE* CVCBStruct::GetVCBStructVariables()
{
	// Get variable list
	return m_pVariableList;
}

void CVCBStruct::SetVCBStructVariables(VARIABLE_TYPE* pVariableList)
{
	// Set variable list
	m_pVariableList = pVariableList;
}

void CVCBStruct::GenerateOutputFile()
{
	// Generate struct header file
	GenerateHeaderFile();
}

void CVCBStruct::GenerateHeaderFile()
{
	int i;
	CStdioFile outputFile;
	CString strOutput;

	// Open output file
	strOutput.Format(_T("output/%s"), m_lpszStructHeaderFile);
	outputFile.Open(strOutput, CFile::modeCreate | CFile::modeNoTruncate | CFile::modeWrite);

	outputFile.SeekToEnd();

	// Write author info
	strOutput.Format(_T("// Author : %s\n"), m_lpszAuthor);
	outputFile.WriteString(strOutput);

	// Write description info
	strOutput.Format(_T("// Description : %s\n"), m_lpszDescription);
	outputFile.WriteString(strOutput);

	// Write struct header
	strOutput.Format(_T("struct %s\n{\n"), m_lpszStructName);
	outputFile.WriteString(strOutput);

	// Write struct members
	for (i=0; i<m_iVariableNumber; i++)
	{
		// Write struct member
		strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
		outputFile.WriteString(strOutput);
	}

	// Write struct footer
	strOutput.Format(_T("\n};\n\n\n"));
	outputFile.WriteString(strOutput);

	// Close output file
	outputFile.Close();
}

void CVCBStruct::SaveObject(CArchive& ar)
{
	int i;

	// Save CVCBStruct object
	ar << m_nObjectType;
	ar << m_dwObjectID;
	ar << m_rcObjectRect;
	ar << (CString)m_lpszStructName;
	ar << (CString)m_lpszStructHeaderFile;
	ar << (CString)m_lpszAuthor;
	ar << (CString)m_lpszDescription;
	ar << m_iVariableNumber;
	for (i=0; i<m_iVariableNumber; i++)
	{
		ar << (CString)m_pVariableList[i].lpszVariableType;
		ar << (CString)m_pVariableList[i].lpszVariableName;
		ar << (CString)m_pVariableList[i].lpszVariableComment;
		ar << m_pVariableList[i].bProperty;
		ar << (CString)m_pVariableList[i].lpszPropertyName;
		ar << m_pVariableList[i].bStatic;
		ar << m_pVariableList[i].variableAccessLevel;
	}
}
